/*
    # ===========================================================================
    # Copyright (C) 2014  Gerhard Hepp
    #
    # This program is free software; you can redistribute it and/or modify it under the terms of
    # the GNU General Public License as published by the Free Software Foundation; either version 2
    # of the License, or (at your option) any later version.
    #
    # This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
    # without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
    # See the GNU General Public License for more details.
    #
    # You should have received a copy of the GNU General Public License along with this program; if
    # not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
    # MA 02110, USA
    # ===========================================================================
*/

#if defined (DEBUG)
 // #define SERIAL
#endif

#if defined (__AVR_ATmega328P__)
#else
#define __AVR_ATmega328P__
#include <avr/iom328p.h>
#endif

#include <stdio.h>

#include <stdint.h>
#include <stdlib.h>
//
#include <avr/interrupt.h>
#include <avr/delay.h>
#include <avr/pgmspace.h>
#include <avr/io.h>

//

#define DEBUG_SERIAL_LN(string) {static const char tx[] PROGMEM = string; serial.serialOutLn_P(tx);}

static const char copyright[] PROGMEM = "copyright Gerhard Hepp 2014";

extern "C"
{
  void
  __cxa_pure_virtual()
  {
    // put error handling here
  }
}

extern volatile uint8_t adc_0[2];
extern volatile uint8_t adc_1[2];
extern volatile uint8_t counter_config[2];
// extern volatile uint8_t counter[2];
extern volatile uint8_t timedcounter[2];

extern volatile uint8_t t0counter;
extern volatile uint16_t t1counter;
extern volatile uint16_t t1TimedCounter;

#define ADC_ENABLE ( 1 << 0 )
#define ADC_REFERENCE ( 1 << 1 )

extern volatile uint8_t adc_0_config;
extern volatile uint8_t adc_1_config;

//
// pure event counter
//
#define CMD_0_COUNTER_MODE  ( 1 << 0 )
#define CMD_0_COUNTER_START ( 1 << 1 )
#define CMD_0_COUNTER_RESET ( 1 << 2 )

//
// Frequency Timer by measuring multiple periods
//
#define CMD_0_TIMER_MODE    ( 1 << 3 )
#define CMD_0_NOISECANCELLER ( 1 << 4 )

//
// Frequency Counter by measuring a fixed (short) time of prox 10ms
//
#define CMD_0_TIMED_COUNTER_MODE ( 1 << 5 )
// Periodendauer. Wenn 0: 10 ms, sonst 20 ms

#define CMD_0_TIMED_COUNTER_MODE_PERIOD ( 1 << 6)
uint8_t old_counter_config[2];

#include "Serial.h"
Serial serial;

#define HARDWARE_20MHz 20
#define HARDWARE_8MHz   8

uint8_t hardware = 0;

#define E_STATUSLED_1_0  { PORTD &= ~(1 << 7);}
#define E_STATUSLED_1_1  { PORTD |=  (1 << 7);}

#define E_STATUSLED_2_0  { PORTB &= ~(1 << 1);}
#define E_STATUSLED_2_1  { PORTB |=  (1 << 1);}

void
statusled1(uint8_t value)
{

 if (value)
          E_STATUSLED_1_1
 else
          E_STATUSLED_1_0
 }
void
statusled2(uint8_t value)
{
  if (value)
      E_STATUSLED_2_1
  else
      E_STATUSLED_2_0
}

uint8_t t_pd4 = 0;
void toggle_PD4(){

#if 0
  if ( t_pd4 == 0 ){
  PORTD |= (1 << 4);
  t_pd4 = 1;
  } else{
      PORTD &= ~(1 << 4);
      t_pd4 = 0;

  }
#else
  PIND |= (1 << 4);
#endif
}

uint8_t analogState = 0;

void
analogConversion()
{
  switch (analogState)
    {

  case 0:
    {
      PRR &= ~(1 << PRADC);
      // sets timing (prescaler = 64) and enables the ADC
      ADCSRA = 0b10000110;
      analogState = 100;
    }
    break;
  case 100:
    {
      if (adc_0_config & ADC_ENABLE)
        {
          // Pullup ausschalten
          PORTC &= ~(1 << 0);
          analogState = 101;
        }
      else
        {
          // hier den Interrupt nicht verbieten, da wenn kein ADC benutzt wird, das zu oft
          // ausgeführt wird.

          adc_0[1] = adc_0[0] = 0xff;
          // Pullup einschalten
          PORTC |= (1 << 0);
          analogState = 110;
        }
    }
    break;
  case 101:
    {
      if (adc_0_config & ADC_REFERENCE)
        {
          // AVCC with external capacitor at AREF pin
          ADMUX = 0b01000000;
        }
      else
        {
          // internal reference voltage
          ADMUX = 0b11000000;
        }

      analogState = 102;
    }
    break;
  case 102:
    {
      // start single conversion #1
      ADCSRA |= (1 << ADSC);
      analogState = 103;
    }
    break;

  case 103:
    {
      if (ADCSRA & (1 << ADSC))
        {
          ;
        }
      else
        {
          analogState = 104;
        }
    }

    break;
  case 104:
    {
      // start single conversion #2
      ADCSRA |= (1 << ADSC);
      analogState = 105;
    }
    break;

  case 105:
    {
      if (ADCSRA & (1 << ADSC))
        {
          ;
        }
      else
        {
          cli();
          adc_0[1] = ADCL;
          adc_0[0] = ADCH;
          sei();
          analogState = 110;
        }
    }

    break;
  case 110:
    {
      if (adc_1_config & ADC_ENABLE)
        {
          // Pullup ausschalten
          PORTC &= ~(1 << 1);
          analogState = 111;
        }
      else
        {
          // hier den Interrupt nicht verbieten, da wenn kein ADC benutzt wird, das zu oft
          // ausgeführt wird.

          adc_1[1] = adc_1[0] = 0xff;
          // Pullup einschalten
          PORTC |= (1 << 1);
          analogState = 120;
        }
    }
    break;

  case 111:
    {
      if (adc_1_config & ADC_REFERENCE)
        {
          // AVCC with external capacitor at AREF pin
          ADMUX = 0b01000001;
        }
      else
        {
          // internal reference voltage
          ADMUX = 0b11000001;
        }

      analogState = 112;
    }
    break;
  case 112:
    {
      // start single conversion #1
      ADCSRA |= (1 << ADSC);
      analogState = 113;
    }
    break;

  case 113:
    {
      if (ADCSRA & (1 << ADSC))
        {
          ;
        }
      else
        {
          analogState = 114;
        }
    }

    break;
  case 114:
    {
      // start second. single conversion
      ADCSRA |= (1 << ADSC);
      analogState = 115;
    }
    break;

  case 115:
    {
      if (ADCSRA & (1 << ADSC))
        {
          ;
        }
      else
        {
          cli();
          adc_1[1] = ADCL;
          adc_1[0] = ADCH;
          sei();
          analogState = 120;
        }
    }

    break;

  case 120:
    {
      // future extension for more channels
      analogState = 0;
    }
    break;
  default:
    {
      analogState = 0;
    }
    break;
    }
}
//
// Die Kommandos aus der SPI-Schnittstelle werden in Betriebsarten umgesetzt.
//
#define VOID_MODE          0
#define TIMER_MODE         1
#define COUNTER_MODE       2
#define TIMED_COUNTER_MODE 4

uint8_t timerMode = VOID_MODE;

#define COUNTER_STATE_INITIAL 0
#define COUNTER_STATE_STOPPED 1
#define COUNTER_STATE_RUNNING 2

uint8_t counterState = COUNTER_STATE_INITIAL;

#define TIMER_STATE_INITIAL 0
#define TIMER_STATE_STOPPED 1
#define TIMER_STATE_RUNNING 2

uint8_t timerState = TIMER_STATE_INITIAL;

#define TIMEDCOUNTER_STATE_INITIAL 0
#define TIMEDCOUNTER_STATE_RUNNING 1

uint8_t timedCounterState = TIMEDCOUNTER_STATE_INITIAL;
// ---------------------------------------------------
// Registers in interrupt for T1, Timer mode
//
volatile uint16_t t1_m0;
volatile uint16_t t1_mN;
volatile uint8_t t1_count;
volatile uint8_t t1_maxCount;

extern volatile uint8_t t1_result[4];

uint8_t noiseCanceller = 0;
uint8_t timedcounter_period = 0;

uint16_t filter[16];
void
enableTimer()
{          // ----------------------------------------
  // ENABLE TIMER
  //
  // input capture pin to  ICP1
  TCCR1A = 0;
  // ICNC1 = 0 Noise canceleor off
  // ICES1 = 1 positive Edge for ICP1
  // clock source = CPU without prescaler
  TCCR1B = 0b01000001 | (noiseCanceller ? (1 << ICNC1) : 0);
  TCCR1C = 0;
  // enable input capture interrupt

  TIMSK1 |= (1 << ICIE1);
  // ---------------------------------------
}
void
disableTimer()
{
  // ----------------------------------------
  // STOP TIMER

  // input capture pin to  ICP1
  TCCR1A = 0;
  // ICNC1 = 0 Noise canceleor off
  // ICES1 = 1 positive Edge for ICP1
  // clock source = CPU without prescaler
  TCCR1B = 0b00000000;
  TCCR1C = 0;
  // enable input capture interrupt

  TIMSK1 &= ~(1 << ICIE1);
  t1_result[3] = 0xff;
}
void
enableCounter()
{

  //-----------------------------------------
  // ENABLE COUNTER
  //
  TCCR1A = 0;
  // External clock, falling edge
  TCCR1B = (1 << CS12) | (1 << CS11) | (1 << CS10);
  if (noiseCanceller)
    TCCR1B |= (1 << ICNC1);
  TCCR1C = 0;

  // Overflow Interrupt
  TIMSK1 |= (1 << TOIE1);
  // --------------------------------------------
}
void
disableCounter()
{
  // --------------------------------------------
  // STOP COUNTER
  //
  TCCR1A = 0;
  // External clock, falling edge
  TCCR1B = 0;
  TCCR1C = 0;

  // disable Overflow Interrupt
  TIMSK1 &= ~(1 << TOIE1);

  // counter[0] = counter[1] = 0xff;
  // --------------------------------------------
}
void
enableTimedCounter()
{
  // ----------------------------------------
  // ENABLE TIMED COUNTER
  //
  TCCR1A = 0;
  // External clock, falling edge
  TCCR1B = 0b00000110;

  TCCR1C = 0;
  // Overflow Interrupt
  TIMSK1 |= (1 << TOIE1);

  // Enable Timer 0 in CTC-Mode
  TCCR0A = 0b00000010;  // Enable CTC-Mode (overflow interrupt and reset in compare to  OCR0A)

  TCCR0B = 0b00000101;  // Tclk / 1024
  // enable overflow interrupt on OCR0A
  TIMSK0 = (1 << OCIE0A);
}
void
disableTimedCounter()
{
  // --------------------------------------------
  // STOP COUNTER
  //
  TCCR1A = 0;
  // External clock, falling edge
  TCCR1B = 0;
  TCCR1C = 0;

  // disable Overflow Interrupt
  TIMSK1 &= ~(1 << TOIE1);
  TIMSK0 &= ~(1 << OCIE0A);
  timedcounter[0] = timedcounter[1] = 0xff;
}
void
manage_timer()
{

  //
  // nur dann Umschaltungen vornehmen, wenn eines der Steuerbits sich aendert
  //
  if ((old_counter_config[1] != counter_config[1])        //
  || (old_counter_config[0] != counter_config[0]))
    {
      uint8_t newTimerMode = VOID_MODE;

      if (counter_config[0] & CMD_0_COUNTER_MODE)
        {
          newTimerMode = COUNTER_MODE;
        }
      else if (counter_config[0] & CMD_0_TIMER_MODE)
        {
          newTimerMode = TIMER_MODE;
        }

      else if (counter_config[0] & CMD_0_TIMED_COUNTER_MODE)
        {
          newTimerMode = TIMED_COUNTER_MODE;
        }
      else
        {
          newTimerMode = VOID_MODE;
        }

      noiseCanceller = 0;
      if (counter_config[0] & CMD_0_NOISECANCELLER)
        noiseCanceller = 1;

      timedcounter_period = 0;
      if (counter_config[0] & CMD_0_TIMED_COUNTER_MODE_PERIOD)
        timedcounter_period = 1;
      //
      // Umschaltung der Betriebsarten, wenn nötig
      //
      switch (timerMode)
        {
      case VOID_MODE:
        {
          switch (newTimerMode)
            {
          case TIMER_MODE:
            {
              enableTimer();
              timerMode = TIMER_MODE;
              timerState = TIMER_STATE_INITIAL;

            }
            break;
          case COUNTER_MODE:
            {
              enableCounter();
              timerMode = COUNTER_MODE;
            }
            break;
          case TIMED_COUNTER_MODE:
            {
              enableTimedCounter();
              timerMode = TIMED_COUNTER_MODE;
              timedCounterState = TIMEDCOUNTER_STATE_INITIAL;
            }
            break;
          case VOID_MODE:
            {
              timerMode = VOID_MODE;
              break;
            }
            }
        }
        break;
      case TIMER_MODE:
        {
          switch (newTimerMode)
            {
          case TIMER_MODE:
            {
            }
            break;
          case COUNTER_MODE:
            {
              disableTimer();
              enableCounter();

              timerMode = COUNTER_MODE;
            }
            break;
          case TIMED_COUNTER_MODE:
            {
              disableTimer();
              enableTimedCounter();
              timerMode = TIMED_COUNTER_MODE;
              timedCounterState = TIMEDCOUNTER_STATE_INITIAL;
            }
            break;
          case VOID_MODE:
            {
              disableTimer();

              timerMode = VOID_MODE;
            }
            break;
            }
        }
        break;
      case COUNTER_MODE:
        {
          switch (newTimerMode)
            {
          case TIMER_MODE:
            {
              disableCounter();
              enableTimer();

              timerMode = TIMER_MODE;
              timerState = TIMER_STATE_INITIAL;

            }
            break;
          case COUNTER_MODE:
            {
            }
            break;
          case TIMED_COUNTER_MODE:
            {
              disableCounter();
              enableTimedCounter();
              timerMode = TIMED_COUNTER_MODE;
              timedCounterState = TIMEDCOUNTER_STATE_INITIAL;
            }
            break;
          case VOID_MODE:
            {
              disableCounter();
              timerMode = VOID_MODE;
            }
            break;
            }
        }
        break;
      case TIMED_COUNTER_MODE:
        {
          switch (newTimerMode)
            {
          case TIMER_MODE:
            {
              disableTimedCounter();
              enableTimer();

              timerMode = TIMER_MODE;
              timerState = TIMER_STATE_INITIAL;

            }
            break;
          case COUNTER_MODE:
            {
              disableTimedCounter();
              enableCounter();
              timerMode = COUNTER_MODE;
            }
            break;
          case TIMED_COUNTER_MODE:
            {
            }
            break;
          case VOID_MODE:
            {
              disableTimedCounter();
              timerMode = VOID_MODE;
            }
            break;
            }
        }
        break;
        }
      old_counter_config[1] = counter_config[1];
      old_counter_config[0] = counter_config[0];
    }
  // ------------------------------------------------------------------------------------------

  switch (timerMode)
    {
  case COUNTER_MODE:
    {
      switch (counterState)
        {
      case COUNTER_STATE_RUNNING:
        {
          if ((counter_config[0] & CMD_0_COUNTER_RESET))
            { //
              // reset counter bit immediateley
              //
              counter_config[0] &= ~CMD_0_COUNTER_RESET;

                {
                  // when timer is running, stop it, reset values
                  // and restart
                  TCCR1B &= ~(1 << CS12) | (1 << CS11) | (1 << CS10) | (noiseCanceller ? (1 << ICNC1) : 0);

                  t1counter = 0;
                  TCNT1 = 0;

                  TCCR1B |= (1 << CS12) | (1 << CS11) | (1 << CS10) | (noiseCanceller ? (1 << ICNC1) : 0);
                }

            }
          if ((counter_config[0] & CMD_0_COUNTER_START)== 0){

            // disable counting
            //
            TCCR1A = 0;
            // External clock, falling edge
            TCCR1B &= ~(1 << CS12) | (1 << CS11) | (1 << CS10) | (noiseCanceller ? (1 << ICNC1) : 0);
            TCCR1C = 0;

            counterState = COUNTER_STATE_STOPPED;
          }
      }
      break;

    case COUNTER_STATE_STOPPED:
      {
          {
            if ((counter_config[0] & CMD_0_COUNTER_RESET))
              { //
                // reset counter bit immediateley
                //
                counter_config[0] &= ~CMD_0_COUNTER_RESET;
                //
                t1counter = 0;
                TCNT1 = 0;
              }
            if ((counter_config[0] & CMD_0_COUNTER_START))
              {

                // enable counting
                //
                TCCR1A = 0;
                // External clock, falling edge
                TCCR1B |= (1 << CS12) | (1 << CS11) | (1 << CS10) | (noiseCanceller ? (1 << ICNC1) : 0);
                TCCR1C = 0;

                counterState = COUNTER_STATE_RUNNING;

              }
          }
        break;

      }
      break;

      }

  }
  break;
case TIMER_MODE:
  {

    if (0)
      {
        for (uint8_t k = 0; k < 3; k++)
          {
            statusled1(0);
            _delay_ms(20);

            statusled1(1);
            _delay_ms(70);
          }
      }

    switch (timerState)
      {
    default:
    case TIMER_STATE_INITIAL:
      {
        // stop timer
        TCCR1B = 0b01000000 | (noiseCanceller ? (1 << ICNC1) : 0);
        ;
        // disable timer capture interrupt
        TIMSK1 &= ~(1 << ICIE1);
        // clear the timer value
        TCNT1 = 0;

        t1_maxCount = 4;
        t1_count = 0;

        // clear Overflow TOV1-Flag by setting a '1' to its bit value (see docu).

        TIFR1 |= (1 << TOV1);
        // clear Capture ICF1-Flag by setting a '1' to its bit value (see docu).
        TIFR1 |= (1 << ICF1);
        // start timer
        TCCR1B = 0b01000001 | (noiseCanceller ? (1 << ICNC1) : 0);

        // enable interrupt
        TIMSK1 |= (1 << ICIE1);
        //
        timerState = TIMER_STATE_RUNNING;
      }
      break;
    case TIMER_STATE_RUNNING:
      {
        if (t1_count == t1_maxCount + 1)
          {
            if (TIFR1 & (1 << TOV1))
              {
                if (t1_maxCount > 3)
                  {

                    t1_maxCount -= 2;

                  }
              }
            else
              {

                uint16_t res = t1_mN - t1_m0;
                uint16_t res2 = 0;
                uint8_t do_filter = 0;
                if (do_filter)
                  {
                    res2 = res;

                    filter[15] = filter[14];
                    filter[14] = filter[13];
                    filter[13] = filter[12];
                    filter[12] = filter[11];
                    filter[11] = filter[10];
                    filter[10] = filter[9];
                    filter[9] = filter[8];
                    filter[8] = filter[7];
                    filter[7] = filter[6];
                    filter[6] = filter[5];
                    filter[5] = filter[4];
                    filter[4] = filter[3];
                    filter[3] = filter[2];
                    filter[2] = filter[1];
                    filter[1] = filter[0];
                    filter[0] = res;

                    uint32_t r = 0;
                    r += (uint32_t) filter[15];
                    r += (uint32_t) filter[14];
                    r += (uint32_t) filter[13];
                    r += (uint32_t) filter[12];
                    r += (uint32_t) filter[11];
                    r += (uint32_t) filter[10];
                    r += (uint32_t) filter[9];
                    r += (uint32_t) filter[8];
                    r += (uint32_t) filter[7];
                    r += (uint32_t) filter[6];
                    r += (uint32_t) filter[5];
                    r += (uint32_t) filter[4];
                    r += (uint32_t) filter[3];
                    r += (uint32_t) filter[2];
                    r += (uint32_t) filter[1];
                    r += (uint32_t) filter[0];

                    res = (uint16_t) ((r / (uint32_t) 16) & 0x0000ffff);
                  }
                uint8_t res_h = (res >> 8) & 0xff;
                uint8_t res_l = (res >> 0) & 0xff;

                cli();
                t1_result[0] = res_h;
                t1_result[1] = res_l;

                t1_result[2] = t1_maxCount;
                t1_result[3] = 0;  // no errors
                sei();

                if (do_filter)
                  {
                    res = res2;
                  }

                //
                // adjust t1_maxcount to get best resolution
                //
                if (t1_maxCount < (255 - 3))
                  {
                    if (res < 0xD000)
                      t1_maxCount += 2;
                  }
                if (t1_maxCount > 3)
                  {
                    if (res > 0xF000)
                      t1_maxCount -= 2;
                  }
              }

            TCCR1B = 0b01000000 | (noiseCanceller ? (1 << ICNC1) : 0);
            // disable timer capture interrupt
            TIMSK1 &= ~(1 << ICIE1);
            // clear the timer value
            TCNT1 = 0;

            t1_count = 0;

            // clear Overflow TOV1-Flag by setting a '1' to its bit value (see doku).

            TIFR1 |= (1 << TOV1);
            // clear Capture ICF1-Flag by setting a '1' to its bit value (see doku).
            TIFR1 |= (1 << ICF1);
            // start timer
            TCCR1B = 0b01000001 | (noiseCanceller ? (1 << ICNC1) : 0);

            // enable interrupt
            TIMSK1 |= (1 << ICIE1);
          }
      }
      break;

      }
  }
  break;
case TIMED_COUNTER_MODE:
  {
#if 0
    toggle_PD4();
#endif
    switch (timedCounterState)
      {
    case TIMEDCOUNTER_STATE_INITIAL:
      {
        if (hardware == HARDWARE_8MHz)
          {
            if ( timedcounter_period == 0 ){
                 // 10ms == 10000 us * 8 / 1024
                 OCR0A = 78;
            }
            else{
                // 20ms == 20000 us * 8 / 1024
                OCR0A = 156;
            }

          }
        else
          {
            OCR0A = 195;
          }
        TCNT0 = 0;
        TCNT1 = 0;
        t0counter = 0;

        timedCounterState = TIMEDCOUNTER_STATE_RUNNING;
#if 0
        PORTD |= (1 << 7);
#endif
      }
      break;
    case TIMEDCOUNTER_STATE_RUNNING:
      {
        if (t0counter > 0)
          {
            cli();
            timedcounter[0] = t1TimedCounter >> 8;
            timedcounter[1] = t1TimedCounter & 0xff;
            sei();
            t0counter = 0;

            // timedCounterState = TIMEDCOUNTER_STATE_RUNNING;
          }

      }
      break;
      }

  }
  break;
  }

}

#define IRC_ENABLE  (  1 << 0 )
#define IRC_START   (  1 << 1 )
#define IRC_DEBUG   (  1 << 2 )


extern uint8_t irc_config;

#define IRC_BLOCK_SIZE 32
#if (IRC_BLOCK_SIZE*2*4 > 256)
#error "value too large"
#endif

uint8_t irc_current_result[2];
uint8_t irc_result_0[IRC_BLOCK_SIZE * 2];
uint8_t irc_result_1[IRC_BLOCK_SIZE * 2];
uint8_t irc_result_2[IRC_BLOCK_SIZE * 2];
uint8_t irc_result_3[IRC_BLOCK_SIZE * 2];

uint8_t old_irc_config;

#define IRC_ENABLED   1
#define IRC_DISABLED  2

uint8_t irc_modus = IRC_DISABLED;
uint8_t new_irc_modus = IRC_DISABLED;

//
// Ergebnis der Erfassung
//
uint8_t irc_status[3];

#define IRC_STATE_STOPPED_0    10
#define IRC_STATE_STOPPED_1    11
#define IRC_WAIT_RUNNING       15
#define IRC_STATE_RUNNING_1    21
#define IRC_STATE_COMPLETED_0  30
#define IRC_STATE_COMPLETED_1  31


uint8_t irc_state;
volatile uint8_t irc_cmd;
uint16_t irc_counter;
uint8_t t2_counter;
uint8_t irc_control = 0;

char buffer[128];

void
print_irc_results()
{
  {

    sprintf(buffer, "have %d bytes", irc_counter);
    serial.serialOutLn(buffer);
  }
  {
    for (uint8_t k = 0; k < IRC_BLOCK_SIZE * 2; k += 2)
      {
        if (k % 16 == 0)
          {
            serial.serialOutLn("");
          }
          {

            sprintf(buffer, "%02x %02x  ", irc_result_0[k + 0], irc_result_0[k + 1]);
            serial.serialOut(buffer);
          }
      }
    serial.serialOutLn("");
  }
  {
    for (uint8_t k = 0; k < IRC_BLOCK_SIZE * 2; k += 2)
      {
        if (k % 16 == 0)
          {
            serial.serialOutLn("");
          }
          {

            sprintf(buffer, "%02x %02x  ", irc_result_1[k + 0], irc_result_1[k + 1]);
            serial.serialOut(buffer);
          }

      }
    serial.serialOutLn("");
  }
  {
    for (uint8_t k = 0; k < IRC_BLOCK_SIZE * 2; k += 2)
      {
        if (k % 16 == 0)
          {
            serial.serialOutLn("");
          }
          {

            sprintf(buffer, "%02x %02x  ", irc_result_2[k + 0], irc_result_2[k + 1]);
            serial.serialOut(buffer);
          }
      }
    serial.serialOutLn("");
  }
  {
    for (uint8_t k = 0; k < IRC_BLOCK_SIZE * 2; k += 2)
      {
        if (k % 16 == 0)
          {
            serial.serialOutLn("");
          }
          {

            sprintf(buffer, "%02x %02x  ", irc_result_3[k + 0], irc_result_3[k + 1]);
            serial.serialOut(buffer);
          }
      }
    serial.serialOutLn("");
  }
}
uint8_t irc_logging_serialnumber = 0;


void
manage_irc()
{

if (old_irc_config != irc_config)
  {

#ifdef SERIAL
    sprintf(buffer, "IRC new config %02x", irc_config);
    serial.serialOut(buffer);
#endif
    if (irc_config & IRC_ENABLE)
      new_irc_modus = IRC_ENABLE;
    else
      new_irc_modus = IRC_DISABLED;

    if (irc_config & IRC_START)
      irc_cmd = 1;
    else if (irc_config & IRC_DEBUG)
      irc_cmd = 4;
    else
      irc_cmd = 0;

    switch (irc_modus)
      {
    case IRC_DISABLED:
      {
        switch (new_irc_modus)
          {
        case IRC_DISABLED:
          {
#ifdef SERIAL
            DEBUG_SERIAL_LN("--> IRC_DISABLED unchanged");
#endif
           }
          break;
        case IRC_ENABLED:
          {
            irc_modus = IRC_ENABLED;
            irc_state = IRC_STATE_STOPPED_0;
            // signal 'no data' by setting line HIGH
            PORTD |= (1 << 4);
#ifdef SERIAL
            DEBUG_SERIAL_LN("--> IRC_ENABLED");
#endif
          }
          break;
          }
      }
      break;
    case IRC_ENABLED:
      {
        switch (new_irc_modus)
          {
        case IRC_DISABLED:
          {
            // signal 'no data' by setting line HIGH
            PORTD |= (1 << 4);

            irc_modus = IRC_DISABLED;
            irc_state = IRC_STATE_STOPPED_0;
            PCMSK2 = 0;
            PCICR &= ~(1 << PCIE2);
            // disable timer
            TIMSK2 = 0;
            TCCR2B = 0;
#ifdef SERIAL
            DEBUG_SERIAL_LN("--> IRC_DISABLED");
#endif
          }
          break;
        case IRC_ENABLED:
          {
#ifdef SERIAL
            DEBUG_SERIAL_LN("--> IRC_ENABLED unchanged");
#endif
          }
          break;
          }
      }
      break;

      }

    old_irc_config = irc_config;
    // on config change, reset flag
        // signal NO result by setting line HIGH
        //
         PORTD |= (1 << 4);
        //
  }

if (irc_modus == IRC_ENABLED)
  {
    switch (irc_state)
      {
    case IRC_STATE_STOPPED_0:
      {
#ifdef SERIAL
        DEBUG_SERIAL_LN("IRC_STATE_STOPPED_0, activate !");
#endif

        irc_state = IRC_STATE_STOPPED_1;

      }
      break;
    case IRC_STATE_STOPPED_1:
      {
        if (irc_cmd & 1)
          {
            // delete the command, when consumed
            irc_cmd = 0;

            //
            // clear buffer
            //
            for (uint8_t k = 0; k < (IRC_BLOCK_SIZE * 2); k++)
                          {
                            irc_result_0[k] = //
                                irc_result_1[k] = //
                                    irc_result_2[k] = //
                                        irc_result_3[k] = 0xff;
                          }

            // signal 'running' by setting line HIGH
            PORTD |= (1 << 4);
#ifdef SERIAL
            {

                sprintf(buffer, "irc_logging_serialnumber: %02x", irc_logging_serialnumber++);
                serial.serialOutLn(buffer);
              }
            DEBUG_SERIAL_LN("IRC_STATE_STOPPED_1, use IRC !");
#endif
            irc_state = IRC_WAIT_RUNNING;

            irc_status[0] = irc_status[1] = irc_status[2] = 0;
            //
            // normal timer mode
            // clear pending interrupt
            TIFR2 |= (1 << TOV2);
            //
            // normal timer mode
            //
            TCCR2A = 0;
            //
            // system clock input, divide by 64
            // at 8MHz this yields to 8 us resolution
            // be aware, that interrupt logic delays decreases accuracy
            //
            TCCR2B = 0b00000100;
            //
            // allow timer 2 overflow interrupt
            TIMSK2 = 0b00000001;
            //
            // setup PIN change Interrupt for PCINT22 aka PD6
            //
            // clear any already pending interrupt
            PCIFR |= (1 << PCIF2);
            PCMSK2 = (1 << PCINT22);
            PCICR |= (1 << PCIE2);
            irc_control = 0;
          }

      }
      break;
    case IRC_WAIT_RUNNING:
      {
        // wait infinitely till first state change, falling edge assumed
        if (irc_control & 1)
          {

            // DEBUG_SERIAL_LN(" start timer, clear register");
            // start timer, clear register
            //
            irc_state = IRC_STATE_RUNNING_1;
            irc_control = irc_counter = 0;
          }
      }
      break;

    case IRC_STATE_RUNNING_1:
      {
        // DEBUG_SERIAL_LN("IRC_STATE_RUNNING_1");
        if (irc_control & 2)
          {
#ifdef SERIAL
            DEBUG_SERIAL_LN("irc_control & 2: timeout");
#endif
            irc_state = IRC_STATE_COMPLETED_0;
            irc_control &= ~2;

            irc_status[1] = (irc_counter / 2) >> 8;
            irc_status[2] = (irc_counter / 2) & 0xff;
            // status zum Schluss setzen,falls zwischendurch gelesen wird
            irc_status[0] = 2;
            break;
          }
        if (irc_control & 1)
          {

            //
            // toggle pin happened
            // Date are MSBh, LSB timer

            uint8_t nBuf = irc_counter / (2 * IRC_BLOCK_SIZE);
            uint8_t iBuf = irc_counter % (2 * IRC_BLOCK_SIZE);
            switch (nBuf)
              {
            case 0:
              irc_result_0[iBuf + 0] = irc_current_result[0];
              irc_result_0[iBuf + 1] = irc_current_result[1];
              break;
            case 1:
              irc_result_1[iBuf + 0] = irc_current_result[0];
              irc_result_1[iBuf + 1] = irc_current_result[1];
              break;
            case 2:
              irc_result_2[iBuf + 0] = irc_current_result[0];
              irc_result_2[iBuf + 1] = irc_current_result[1];
              break;
            case 3:
              irc_result_3[iBuf + 0] = irc_current_result[0];
              irc_result_3[iBuf + 1] = irc_current_result[1];
              break;
              }

            irc_counter += 2;

            if (irc_counter >= (4 * 2 * IRC_BLOCK_SIZE))
              {
#ifdef SERIAL
                DEBUG_SERIAL_LN("irc_counter == MAX, too many edges");
#endif
                irc_state = IRC_STATE_COMPLETED_0;

                irc_status[1] = (irc_counter / 2) >> 8;
                irc_status[2] = (irc_counter / 2) & 0xff;
                // status zum Schluss setzen,falls zwischendurch gelesen wird
                irc_status[0] = 1;
              }
            irc_control &= ~1;
            break;
          }

      }
      break;

    case IRC_STATE_COMPLETED_0:
      {
        // signal result by setting line LOW
        PORTD &= ~(1 << 4);

#ifdef SERIAL
        {

                       sprintf(buffer, "irc_logging_serialnumber: %02x", irc_logging_serialnumber++);
                       serial.serialOutLn(buffer);
                     }
        DEBUG_SERIAL_LN("IRC_STATE_COMPLETED_0");
#endif
        // disable pin change interrupt
        PCMSK2 = 0;
        PCICR &= ~(1 << PCIE2);
        // disable timer
        TIMSK2 = 0;
        TCCR2B = 0;
        //
        irc_state = IRC_STATE_COMPLETED_1;

      }
      break;
    case IRC_STATE_COMPLETED_1:
      {
#ifdef SERIAL
        DEBUG_SERIAL_LN("IRC_STATE_COMPLETED_1 and waiting to send data");
        print_irc_results();
#endif
        irc_state = IRC_STATE_STOPPED_0;
      }
      break;

      }
  }
}

int
main()
{
  {
    hardware = HARDWARE_8MHz;

    adc_0_config = 0b00000000;
    adc_1_config = 0b00000000;
    analogState = 0;

    counter_config[0] = 0;
    counter_config[1] = 0;   //

// when timer bits are changing, then manage them.
// set them to 0, so manage_code is starting counting.
    old_counter_config[0] = 0;
    old_counter_config[1] = 0;

    t1_result[0] = t1_result[1] = t1_result[2] = t1_result[3] = 0x00;

    irc_config = 0;
    old_irc_config = 0;

//  status = 0;

// -----------------------------------------------------
// PORTB
//    PB.4 ist MISO
//    PB.0 ist ICP1
//    PB.1 ist LED
    DDRB = 0b00010010;
    PORTB = 0b11111111;   // enable pull up for not needed inputs

// -----------------------------------------------------
// PORT C

    DDRC = 0b00000000;
// hier liegen zwei AD-Wandler
    PORTC = 0b11111100;   // enable pull up for not needed inputs

// -----------------------------------------------------
// PORTD

// Pin 7 ist LED-Ausgang
// Pin 4 ist Signal Ausgang (pullup enabled)
//
// die LED sind  gegen Masse angeschlossen
//
    PORTD = 0b11111111;

    DDRD = 0b10010000;

// -------------------------------------------------
// enable pin change interrupt, used to reset the SPI logic
//
    PCICR |= (1 << PCIE0);

    PCMSK0 |= (1 << PCINT2);

// -------------------------------------------------
// no power savings

    PRR = 0;

// -------------------------------------------------
// enable SPI
//
    SPCR = (1 << SPIE) | (1 << SPE);

//
    sei();

#ifdef SERIAL
    serial.init();
    DEBUG_SERIAL_LN("steckbrett_328 2014-05-30 1.7.6");
#endif

    for (uint8_t i = 0; i < hardware; i++)
      {
        statusled1(1);
        statusled2(1);

        for (int k = 0; k < 150; k++)
          {
            manage_timer();
            analogConversion();
            manage_irc();
            if (hardware == HARDWARE_8MHz)
              _delay_ms(1);
            else
              _delay_ms(2);
          }

        statusled1(0);
        statusled2(0);

        for (int k = 0; k < 150; k++)
          {
            manage_timer();
            analogConversion();
            manage_irc();
            if (hardware == HARDWARE_8MHz)
              _delay_ms(1);
            else
              _delay_ms(2);
          }
      }
    uint8_t toggle = 0;
    while (1)
      {
        manage_timer();
        analogConversion();
        manage_irc();

        if (0)
          {
            if (toggle)
              {
                statusled1(1);
                toggle = 0;
              }
            else
              {
                statusled1(0);
                toggle = 1;
              }
          }

      }

    return 0;
  }
}
