# -*- coding: utf-8 -*-
    # ===========================================================================
    # Copyright (C) 2014  Gerhard Hepp
    #
    # This program is free software; you can redistribute it and/or modify it under the terms of
    # the GNU General Public License as published by the Free Software Foundation; either version 2
    # of the License, or (at your option) any later version.
    #
    # This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
    # without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
    # See the GNU General Public License for more details.
    #
    # You should have received a copy of the GNU General Public License along with this program; if
    # not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
    # MA 02110, USA
    # ===========================================================================

 
import time
import RPi.GPIO as GPIO
import programmer.Programmer

import spidev

            
class Peripherie:
    debug = False

    spi_bus = 0
    spi_device = 0
    # GPIO Pin Numbers (BMC-Nummerierung)

    RESET = 24
    IRC_SIGNAL = 23
    
    # Kommandos für den Prozessor Atmega 238
    GET_VERSION = 0x80

    GET_ADC_CONFIG = 0x91
    SET_ADC_CONFIG = 0x92
    
    GET_TESTDATA = 0x93
    
    GET_ADC_0 = 0x81
    GET_ADC_1 = 0x82
    
    GET_COUNTER = 0x83
    GET_TIMER = 0x87
    
    SET_COUNTER_CONFIG = 0x85
    GET_COUNTER_CONFIG = 0x86
    GET_TIMEDCOUNTER = 0x89
    
    SET_LED_0 = 0x84
    SET_LED_1 = 0x88
    
    SET_IRC_CONFIG = 0x8a
    
    GET_IRC_DATA_0 = 0x8b
    GET_IRC_DATA_1 = 0x8c
    GET_IRC_DATA_2 = 0x8d
    GET_IRC_DATA_3 = 0x8e
    
    GET_IRC_STATUS = 0x8f
    
    def __init__(self, bus=0, device=0, spi_speed=240000):
        
        GPIO.setmode(GPIO.BCM)
        GPIO.setwarnings(False)
        
        GPIO.setup(self.RESET, GPIO.OUT)
        GPIO.output(self.RESET, GPIO.HIGH)

        GPIO.setup(self.IRC_SIGNAL, GPIO.IN)
        
        self.spi = spidev.SpiDev()
        self.spi.open(self.spi_bus, self.spi_device)
            
        self.spi.max_speed_hz = spi_speed
        
    def close(self):
        self.spi.close()

    def rst(self, value):
        GPIO.output(self.RESET, not(value) )
    def irc_signal(self):
        return GPIO.input(self.IRC_SIGNAL)
    
    #
    # CPU-command
    #
    # rückgabe wert 0xff: erfolgreich, Platz im Puffer
    # rückgabe wert 0xfe: erfolgreich, kein Platz mehr im Puffer
    # rückgabe wert 0xfd: nicht erfolgreich, da kein Platz im Puffer

    def cpu_cmd_r(self, l):
        if self.debug:
            print("cpu_cmd_r: ")
            for i in range(len(l)):
                print("{0:02x} ".format(l[i]))
        # speed so langsam setzen, damit interrupt genuegend zeit hat
                
        r = self.spi.xfer2(list(l))
        return r

    def cpu_cmd(self, l):
        
        # speed so langsam setzen, damit interrupt genuegend zeit hat
        
        r = self.spi.xfer2(list(l))
    
    #
    #
    # ############################# PROGRAMMIERUNG
    #                
    #
    def program_reset_on(self):
        """callback method for programming"""
        self.rst(True)
    
    def program_reset_off(self):
        """callback method for programming"""
        self.rst(False)

    def program_spi_xfer(self, data):
        """callback method for programming"""
        self.spi.max_speed_hz = 200000
        r = self.spi.xfer2(data)
        return r
    
    def program_init(self):
        """special initialization code (e.g. switch off other hardware) """
        self.program_reset_on()
        pass

    def program_close(self):
        self.program_reset_off()

    def readCode(self, fileName):
        self.program_init()
        prog = programmer.Programmer.Programmer(self)
        prog.programming_readCode(fileName)
        self.program_close()

    def readFuses(self):
        self.program_init()
        prog = programmer.Programmer.Programmer(self)
        prog.programming_readFuses()
        self.program_close()

    def writeFuse(self, value):
        """write fuse 'low fuse'"""
        self.program_init()
        prog = programmer.Programmer.Programmer(self)
        prog.programming_writeFuse(value)
        self.program_close()

    def  writeCode(self, fileName):
        self.program_init()
        prog = programmer.Programmer.Programmer(self)
        prog.programming_writeCode(fileName)
        self.program_close()

    def clearCode(self):
        self.program_init()
        prog = programmer.Programmer.Programmer(self)
        prog.programming_clearCode()
        self.program_close()
       
    # ################################################
    
    def cpu_led_on(self):
        if self.debug:
            print("cpu_led_on")
        
        self.cpu_cmd(list([self.SET_LED_1]))
        
    def cpu_led_off(self):
        if self.debug:
            print("cpu_led_off")
        
        self.cpu_cmd(list([self.SET_LED_0]))
        
    def cpu_get_version(self):
        if self.debug:
            print("cpu_get_version")
        
        r = self.cpu_cmd_r(list([self.GET_VERSION, 0, 0]))
        return r

    def cpu_get_counter(self):
        if self.debug:
            print("cpu_get_counter")
        
        r = self.cpu_cmd_r(list([self.GET_COUNTER, 0, 0, 0, 0]))
        return r
    
    def cpu_get_timer(self):
        if self.debug:
            print("cpu_get_timer")
        
        r = self.cpu_cmd_r(list([self.GET_TIMER, 0, 0, 0, 0]))
        return r
    
    def cpu_get_timedcounter(self):
        if self.debug:
            print("cpu_get_timedcounter")
        
        r = self.cpu_cmd_r(list([self.GET_TIMEDCOUNTER, 0, 0]))
        return r
        
    def cpu_get_testdata(self):
        if self.debug:
            print("cpu_get_testdata")
        
        r = self.cpu_cmd_r(list([ self.GET_TESTDATA, 0, 0, 0, 0, 0, 0, 0]))
        return r
    
    def cpu_wait_irc_result(self, timeout):
        for _ in range(0, timeout*100):
            if self.irc_signal() == 0:
                break;
            time.sleep(0.01)
                
    def cpu_get_irc_data(self):
        if self.debug:
            print("cpu_irc_data")
        
        e = []
        for _ in range(64):
            e.append(0);
        
        l = [ self.GET_IRC_DATA_0 ]
        l.extend(e);
        r0 = self.cpu_cmd_r( list( l ))
        
        l = [ self.GET_IRC_DATA_1 ]
        l.extend(e);
        r1 = self.cpu_cmd_r( list( l ))

        l = [ self.GET_IRC_DATA_2 ]
        l.extend(e);
        r2 = self.cpu_cmd_r( list( l ))
        
        l = [ self.GET_IRC_DATA_3 ]
        l.extend(e);
        r3 = self.cpu_cmd_r( list( l ))

        r = []
        
        r.extend(r0[1:])
        r.extend(r1[1:])
        r.extend(r2[1:])
        r.extend(r3[1:])
        
        return r

    def cpu_get_adc_config(self):
        if self.debug:
            print("cpu_get_adc_config")
        
        r = self.cpu_cmd_r(list([self.GET_ADC_CONFIG, 0, 0]))
        return r

    def cpu_set_adc_config(self, c0, c1):
        if self.debug:
            print("cpu_set_adc_config", c0, c1)
        
        r = self.cpu_cmd_r(list([self.SET_ADC_CONFIG, c0, c1]))
        return r

    def cpu_set_counter_config(self, c0, c1):
        if self.debug:
            print("cpu_set_counterconfig", c0, c1)
        
        r = self.cpu_cmd_r(list([self.SET_COUNTER_CONFIG, c0, c1]))
        return r

    def cpu_set_irc_config(self, c0):
        if self.debug:
            print("cpu_set_irc_config", c0)
        
        r = self.cpu_cmd_r(list([self.SET_IRC_CONFIG, c0 ]))
        return r

    def cpu_get_counter_config(self):
        if self.debug:
            print("cpu_get_counterconfig")
        
        r = self.cpu_cmd_r(list([self.GET_COUNTER_CONFIG, 0, 0 ]))
        return r

    def cpu_get_adc_0 (self):
        if self.debug:
            print("cpu_get_adc_0")
        
        r = self.cpu_cmd_r( list( [ self.GET_ADC_0, 0, 0]))
        return r
    
    def cpu_get_adc_1 (self):
        if self.debug:
            print("cpu_get_adc_1")
        
        r = self.cpu_cmd_r( list( [self.GET_ADC_1, 0, 0]))
        return r
