# -*- coding: utf-8 -*-
    # ===========================================================================
    # Copyright (C) 2014  Gerhard Hepp
    #
    # This program is free software; you can redistribute it and/or modify it under the terms of
    # the GNU General Public License as published by the Free Software Foundation; either version 2
    # of the License, or (at your option) any later version.
    #
    # This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
    # without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
    # See the GNU General Public License for more details.
    #
    # You should have received a copy of the GNU General Public License along with this program; if
    # not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
    # MA 02110, USA
    # ===========================================================================

from __future__ import print_function
                
import time
import sys

import peripherie


def decodeRC5(irc):
    ll = 0
    for i in range(0, len(irc), 2):
        if irc[i] == 255 and irc[i+1]== 255:
            break
        ll += 2
    
    # check range
    error = ''
    for i in range (0, ll, 2):
        val = irc[i] * 256 + irc[i+1]
        if val < 100:
            error = 'value smaller than 100'
        if val > 240:
            error = 'value larger than 240'
        if 125 < val < 200:
            error = 'value larger than 125, less than 200'
    if error != '':
        print (error)
        return
                
    res = '1'
    state = '1'
    
    for i in range (0, ll, 2):
        val = irc[i] * 256 + irc[i+1]
        if val < 175:
            x = 'S'
        else:
            x = 'L'
        newState = state
        
        if state == '1':
            if x == 'S':
                newState = '1s'
            if x == 'L':
                newState = '0'
                res += '0'
        elif state == '1s':
            if x == 'S':
                newState = '1'
                res += '1'
            if x == 'L':
                newState = 'error'
        elif state == '0':
            if x == 'S':
                newState = '0s'
            if x == 'L':
                newState = '1'
                res += '1'
        elif state == '0s':
            if x == 'S':
                newState = '0'
                res += '0'
            if x == 'L':
                newState = 'error'
        elif state == 'error':
            print ("error" )
            break;
        
        if debug:
            print("{s:s} -- [ {x:s} ] --> {ns:s}".format(s=state, ns=newState, x=x ))
            print("    {b:s}".format(b=res))
        
        state = newState
        
    if len(res) != 14:
        print("error, not exactly 14 Bits: ", res)
    else:
        print("RC5", res)            


peri = peripherie.Peripherie(spi_speed=240000)


debug = False

try:

    peri.cpu_led_on()
    
    if False:
        peri.rst(1)
        time.sleep(0.1)
        peri.rst(0)
        time.sleep(5)

    # in case it is alread running
    peri.cpu_set_irc_config(0)
    time.sleep(0.1)
    
    # activate
    peri.cpu_set_irc_config(1)
    time.sleep(0.1)
    # .. and start aquisition
    peri.cpu_set_irc_config(3)
    
    time.sleep(0.1)
    print("set config = 3, press button now ! you have 60 sec time")
    
    peri.cpu_wait_irc_result(60)
    
    irc = peri.cpu_get_irc_data()

    peri.cpu_led_off()
    
    if debug:
        print (irc)
    
    for i in range(0,128):
        if i % 32 == 0:
            print("")
        if i % 8 == 0:
            print("")
        print("{x0:02x} {x1:02x}  ". format(x0=irc[i*2+0], x1=irc[i*2+1])  , end='')
    
    print("")
    print("")
    # let the 328 print out its internal buffer.
    #
    ## peri.cpu_set_irc_config(5)
    if False:
        #
        # print data suitable for excel display
        #
        toggle = 1
        t0 = 0
        
        print ("{v:d};{val:d}".format(v=toggle,val=t0))
        toggle = 0
        print ("{v:d};{val:d}".format(v=toggle,val=t0))
            
        for i in range(0, len(irc), 2):
            if irc[i] == 255 and irc[i+1]== 255:
                break
            if irc[i] == 0 and irc[i+1]== 0:
                break
            
            val = irc[i] * 256 + irc[i+1]
            t0 += val
            print ("{v:d};{val:d}".format(v=toggle,val=t0))
            if toggle == 1:
                toggle = 0
            else:
                toggle = 1
            print ("{v:d};{val:d}".format(v=toggle,val=t0))
    
    if True:
        decodeRC5(irc)
                
        
except KeyboardInterrupt:
    print("interrupt")
except:
    print ("Unexpected error:", str(sys.exc_info()[0]))
    print("error")
    raise    
